#include <BF_Parameters.h>
#include <SensorDefinitions.h>

/*

Blackfin Fingerprint sensor project - Rev 1.0

This software-code sets up the interface between the ADSP-BF533 Blackfin processor and the 
Atmel AT77C104B sensor. It contains the command-interface where different commands are passed
to the sensor in order to receive pixel data. This raw data is re-arranged to get a complete
fingerprint.

*/

extern void PLLSetupBF(void);				//Sets up the Core Clock and System clock for BF
extern void SetupInterruptsBF(void);		//Sets up the interrupts for SPI and PF4 (IRQ)
extern void SetUpSlaveSelects(void); 		//Configure PF1, PF2 as output flags and drive them logic HIGH
extern void ResetSensor(void);		  		//Configure PF3 as an output, drive HIGH for at least 10usec and provide delay
extern void SetUpAndEnableSlowSPI(void);	//Configure and enable SPI, drive SSS/ LOW and provide delay
extern void SetUpAndEnableFastSPI(void);	//Configure and enable SPI, drive FSS/ LOW and provide delay
extern void SetupDMA(void);
extern void DisableSlowSPI(void);			//Check SPI FIFO status (empty), SPIF (transfer completed), Disable SPI and de-activate SSS/
extern void DisableFastSPI(void);			//Disable SPI
extern void WriteSensorRegister(int CommandAndAddress, int value);		//
extern bool ReadSensorRegister(int CommandAndAddress);
extern void SetUpCommandArray(int CommandAndAddress, int value); 
extern void DelayMin1SckCycle(void);		//This is to take care of Trstsu, Tsssu
extern void RearrangeData(void);


/*
Define Globals
*/
char CommandArray[5];
char SensorDataArray[5];
char SPITransferStatus;
char SPITransferCount;
char SensorRequest = 1;			//0: Sensor request; 1: No sensor request
char N;
char Data[FN][233][4];
char DataProcessed[FN][8][232];
char COREorDMA;					//0: Core; 1: DMA
char ProcessData;				//0: Request for process data; 1: No request for process data
bool ReadError;					//0: Read Error not occured, 1: Read Error occured
char DummyRead;

void main(void)
{
    while(1)
    {
   
    COREorDMA = 0;
    ReadError = 0;
    ProcessData = 1;
    PLLSetupBF();
    SetupInterruptsBF();
        
    SetUpSlaveSelects();
    ResetSensor();
    


    WriteSensorRegister(ModeCtrlWrite, 0);			//MODECTRL register - Ocillator Active
   	    
   	WriteSensorRegister(ClickCtrlWrite, CLICKFREQ_90 | CLICKDET_01 | CLICKCPT_20);	//CLICKCTRL register 
    WriteSensorRegister(NavCtrlWrite, NAVFREQ_2900 | NAVV_01 | CLICKV_01); 		//NAVCTRL register - Enter Clickv value
   	WriteSensorRegister(MovCtrlWrite, 0x09); 		//MOVCTRL register
   	
   	WriteSensorRegister(ModeCtrlWrite, MODE_CLICK); 		//MODECTRL register - Enter Click mode
   	WriteSensorRegister(EnctrlWrite, CLICKEN); 		//ENCTRL register - Enable interrupts on Click 
   	
   	printf("Waiting for a click...\n");
   	  	
  	SensorRequest = 1;
   	while(SensorRequest);
   	
  	ReadSensorRegister(StatusRead);
   	WriteSensorRegister(ModeCtrlWrite, MODE_NAV); 		//MODECTRL register - Enter Navigation mode
   	WriteSensorRegister(EnctrlWrite, MOVEN);			//ENCTRL register - Enable interrupts on Movement 
   	
   	printf("Waiting for movement...\n");    
   	SensorRequest = 1;
	   	while(SensorRequest);
    
   
   	ReadSensorRegister(StatusRead);
   	WriteSensorRegister(ModeCtrlWrite, MODE_STANDBY);	//MODECTRL register - Enter Standby mode
   	WriteSensorRegister(ModeCtrlWrite, MODE_ACQ);		//MODECTRL register - Enter Acquisition mode
   	   
   	WriteSensorRegister(HeatCtrlWrite, HEAT | WDOGEN | HEATV_100);
   	WriteSensorRegister(EnctrlWrite, READERREN);
   
    SetupDMA();
    SetUpAndEnableFastSPI();
   
    printf("Acquiring...\n");
    
    while(ProcessData);
    printf("Acquisition complete!\n");
    
          
    WriteSensorRegister(HeatCtrlWrite, 0x00);
    WriteSensorRegister(ModeCtrlWrite, MODE_STANDBY);
    
    if(SensorRequest & ReadSensorRegister(StatusRead))
    {
        printf("Read Error - Quitting operation\n");
    }
    else
    {
    	RearrangeData();
    	printf("Re-arrangement complete!!\n");
    }
    
	}
}


